﻿Public Class PSE
    Private Const tPowerUp As Double = 500

    ' Form a power on waveform with visible slew
    Private powerOnWaveform(tPowerUp) As Double

    Public Enum pseState
        backoff
        detect
        tpon
        powerUp ' time extended to allow user to see if other Alt eventually enabled
        powerOn ' Used to know sim is complete
    End Enum

    Public Enum pseType
        endspan
        midspan
    End Enum

    Private myTpon As Integer = 400
    Private myTdetect As Integer = 300

    Private myState As pseState
    Private myCounter As Integer
    Private myType As pseType
    Private sharedGenerator As System.Random

    Public ReadOnly Property getState() As pseState
        Get
            Return myState
        End Get
    End Property

    Public Sub New(ByVal type As pseType, _
                   ByVal backoff As Integer, _
                   ByVal tpon As Integer, _
                   ByVal tdetect As Integer, _
                   ByRef randGenerator As System.Random)
        ' Assign identity as private object variables
        myState = pseState.backoff
        myType = type
        myTpon = tpon
        myTdetect = tdetect
        sharedGenerator = randGenerator

        ' 1...N ms delay to first detect
        myCounter = sharedGenerator.Next(1, backoff)

        ' Form a power on waveform with visible slew
        powerOnWaveform(0) = -10
        powerOnWaveform(1) = -20
        powerOnWaveform(2) = -30
        powerOnWaveform(3) = -40
        For i As Integer = 4 To tPowerUp
            powerOnWaveform(i) = -50
        Next

    End Sub

    Public Function tick() As Double
        If myCounter > 0 Then
            ' Still counting down in state, 
            ' Return appropriate voltage based on state
            myCounter -= 1
            Select Case myState
                Case pseState.backoff
                    Return 0 ' Volts
                Case pseState.detect
                    Return -10
                Case pseState.tpon
                    Return 0
                Case pseState.powerUp
                    Return powerOnWaveform(tPowerUp - myCounter)
                Case Else ' pseState.powerUp
                    Return -50
            End Select

        Else
            ' Goto Next State
            ' Preform countdown value
            ' Return appropriate voltage based on next state
            Select Case myState
                Case pseState.backoff
                    myState = pseState.detect
                    myCounter = myTdetect
                    Return -10
                Case pseState.detect
                    myState = pseState.tpon
                    If myTpon = 0 Then
                        ' No tpon state, skip it...
                        myState = pseState.powerUp
                        myCounter = tPowerUp - 1
                    Else
                        myCounter = sharedGenerator.Next(1, myTpon)
                        Return 0
                    End If
                Case pseState.tpon
                    myState = pseState.powerUp
                    myCounter = tPowerUp - 1
                    Return powerOnWaveform(0)
                Case pseState.powerUp
                    myState = pseState.powerOn
                    Return -50
                Case Else ' pseState.powerOn
                    Return -50
            End Select
        End If
    End Function

    Public Sub restart(ByVal backoff As Integer)
        myState = pseState.backoff

        myCounter = backoff
    End Sub
End Class
